/*
 * handy tech braille star 80 emulator
 * Authro: Marco Skambraks <marco@ammec.de>
 *  SuSE Linux Products GmbH Nuernberg
 *
 *
 * This is free software, placed under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation.  Please see the file COPYING for details.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <ncurses.h>
#include <string.h>

#include "brlemu.h"

/* communication functions */
void send_id (void);
void read_data (void);
void send_cmd (const unsigned char *cmd);

/* helper functions */
int processkeys (void);
unsigned char recode (unsigned char code);
void wprint (int x, int y, const char *str);

int devfd = 0;
WINDOW *mywin;

int main (int argc, char **argv)
{

  struct termios tiodata;

  if (argc < 2)
   {
     fprintf (stderr, "usage: %s /dev/ttyS0 | /dev/ttyUSB0\n", argv[0]);
     return 1;
   }

  devfd = open (argv[1], O_RDWR | O_NOCTTY);
  if (devfd < 0 || tcgetattr (devfd, &tiodata))
   {
     if (devfd > 0)
       close (devfd);

     fprintf (stderr, "%s: error opening %s\n", argv[0], argv[1]);
     return 2;
   }

  tiodata.c_cflag = (CLOCAL | PARODD | PARENB | CREAD | CS8);
  tiodata.c_iflag = IGNPAR;
  tiodata.c_oflag = 0;
  tiodata.c_lflag = 0;
  tiodata.c_cc[VMIN] = 0;
  tiodata.c_cc[VTIME] = 0;

  /* force DTR down, flush any pending data and
   * set the new configuration
   */
  if (cfsetispeed (&tiodata, B0) ||
      cfsetospeed (&tiodata, B0) ||
      tcsetattr (devfd, TCSANOW, &tiodata) ||
      tcflush (devfd, TCIOFLUSH) ||
      cfsetispeed (&tiodata, B19200) ||
      cfsetospeed (&tiodata, B19200) || tcsetattr (devfd, TCSANOW, &tiodata))
   {
     /* --fake is e.g. for qemu that provides a "fake" serial port
      * do not complain in this case
      */
     if (!(argc > 2 && strcmp(argv[2], "--fake") == 0))
      {
        fprintf (stderr, "%s: init failed\n", argv[0]);
        return 3;
      }
   }

  /* wait - and send the braille ID */
/*  usleep (1000);
  send_id ();
  */

/* init the ncurses stuff */
  initscr ();
  noecho ();
  keypad (stdscr, true);
  mywin = subwin (stdscr, 25, 80, 0, 0);
  syncok (mywin, true);
  nodelay (stdscr, true);

  wprint (36, 2, "braille star 80 emulator");
  wprint (0, 8,
	  "up/down = brl up/down - pgup/pgdown = first/last line - enter = jump to focus");
  wprint (0, 9, "a = atributes off - q = quit");

  while (processkeys ())
   {
     usleep (5000);
     read_data ();

   }

  delwin (mywin);
  endwin ();
  close (devfd);
  return 0;
}
unsigned char recode (unsigned char code)
{

  switch (code)
   {
   case 0:
     return ' ';
   case 1:
     return 'a';
   case 65:
     return 'A';
   case 3:
     return 'b';
   case 67:
     return 'B';
   case 9:
     return 'c';
   case 73:
     return 'C';
   case 25:
     return 'd';
   case 89:
     return 'D';
   case 17:
     return 'e';
   case 81:
     return 'E';
   case 11:
     return 'f';
   case 75:
     return 'F';
   case 27:
     return 'g';
   case 91:
     return 'G';
   case 19:
     return 'h';
   case 83:
     return 'H';
   case 10:
     return 'i';
   case 74:
     return 'I';
   case 26:
     return 'j';
   case 90:
     return 'J';
   case 5:
     return 'k';
   case 69:
     return 'K';
   case 7:
     return 'l';
   case 71:
     return 'L';
   case 13:
     return 'm';
   case 77:
     return 'M';
   case 29:
     return 'n';
   case 93:
     return 'N';
   case 21:
     return 'o';
   case 85:
     return 'O';
   case 15:
     return 'p';
   case 79:
     return 'P';
   case 31:
     return 'q';
   case 95:
     return 'Q';
   case 23:
     return 'r';
   case 87:
     return 'R';
   case 14:
     return 's';
   case 78:
     return 'S';
   case 30:
     return 't';
   case 94:
     return 'T';
   case 37:
     return 'u';
   case 101:
     return 'U';
   case 39:
     return 'v';
   case 103:
     return 'V';
   case 58:
     return 'w';
   case 122:
     return 'W';
   case 45:
     return 'x';
   case 109:
     return 'X';
   case 61:
     return 'y';
   case 125:
     return 'Y';
   case 53:
     return 'z';
   case 117:
     return 'Z';

   case 156:
     return '';
   case 112:
     return '';
   case 170:
     return '';
   case 148:
     return '';
   case 179:
     return '';
   case 166:
     return '';
   case 188:
     return '';
   case 40:
     return '$';
   case 120:
     return '_';
   case 76:
     return '\\';
   case 12:
     return '|';
   case 46:
     return '~';
   case 28:
     return '`';

   case 63:
     return '%';
   case 47:
     return '&';
   case 44:
     return '0';
   case 33:
     return '1';
   case 35:
     return '2';
   case 41:
     return '3';
   case 57:
     return '4';
   case 49:
     return '5';
   case 43:
     return '6';
   case 59:
     return '7';
   case 51:
     return '8';
   case 42:
     return '9';

   case 18:
     return ':';
   case 4:
     return '.';
   case 2:
     return ',';
   case 36:
     return '-';
   case 34:
     return '?';
   case 16:
     return '!';
   case 32:
     return '\'';
   case 92:
     return '@';
   case 6:
     return ';';
   case 20:
     return '*';
   case 22:
     return '+';
   case 54:
     return '=';
   case 8:
     return '\"';
   case 60:
     return '#';
   case 50:
     return '/';
   case 24:
     return '>';
   case 55:
     return '{';
   case 62:
     return '}';
   case 119:
     return '[';
   case 126:
     return ']';
   case 38:
     return '(';
   case 52:
     return ')';
   case 0xff:
     return '%';
   default:
     return 0xff;
   }
}
void wprint (int x, int y, const char *str)
{
  wmove (mywin, y, x);
  waddstr (mywin, str);
  wrefresh (mywin);
}
void send_cmd (const unsigned char *cmd)
{
  int i;
  unsigned char tmpcmd = 0;

/* press keys */
  for (i = 0; cmd[i]; i++)
   {
     tmpcmd = cmd[i];
     write (devfd, &tmpcmd, 1);
     usleep (500);
   }

/* release keys */
  for (--i; i > -1; i--)
   {
     tmpcmd = cmd[i] | 0x80;
     write (devfd, &tmpcmd, 1);
     usleep (500);
   }

}

void read_data (void)
{
  unsigned char code = 0;
  unsigned char brldsp[81];
  int i = 0;

  if (read (devfd, &code, 1) > 0)
   {
     if (code == 1)
      {
	memset (brldsp, 0, 81);
	while (i < 80)
	 {
	   i++;
	   usleep (1000);
	   read (devfd, &code, 1);
	   if (recode (code) != 0xff)
	     brldsp[i - 1] = recode (code);
	   else
	    {
	      brldsp[i - 1] = ' ';
	    }
	 }

	wprint (0, 4, (const char *) brldsp);
	move (4, 0);
	i = 0;
/*	usleep (5000);
	code = 0x7e;
	write (devfd, &code, 1);*/
      }
     else if (code == ID_REQ)
      {
	send_id ();
      }

   }

}

void send_id (void)
{
  unsigned char code = ID_CMD;

  write (devfd, &code, 1);
  usleep (1000);
  code = ID;
  write (devfd, &code, 1);
  code = 0;
}

int processkeys ()
{
  int tmpkey = 0;
  unsigned char cmdvec[10] = "";

  memset (cmdvec, 0, sizeof (cmdvec));

  switch ((tmpkey = getch ()))
   {

   case -1:
     break;
   case 'a':
     cmdvec[0] = B3;
     break;
   case 'q':
     return 0;
   case KEY_PPAGE:
     cmdvec[0] = B2;
     cmdvec[1] = B5;
     break;

   case KEY_NPAGE:
     cmdvec[0] = B2;
     cmdvec[1] = B6;
     break;
   case KEY_ENTER:
   case 10:
     cmdvec[0] = B1;
     break;
   case KEY_UP:
     cmdvec[0] = B5;
     break;
   case KEY_DOWN:
     cmdvec[0] = B6;
     break;

   }
  if (cmdvec[0])
    send_cmd (cmdvec);
  return 1;
}
